package net.pterodactylus.sone.template;

import static java.lang.String.valueOf;
import static net.pterodactylus.sone.utils.NumberParsers.parseInt;

import java.io.IOException;
import java.io.StringReader;
import java.io.StringWriter;
import java.io.UnsupportedEncodingException;
import java.io.Writer;
import java.net.URLEncoder;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import net.pterodactylus.sone.core.Core;
import net.pterodactylus.sone.data.Album;
import net.pterodactylus.sone.data.Sone;
import net.pterodactylus.sone.text.AlbumPart;
import net.pterodactylus.sone.text.FreenetLinkPart;
import net.pterodactylus.sone.text.LinkPart;
import net.pterodactylus.sone.text.Part;
import net.pterodactylus.sone.text.PartContainer;
import net.pterodactylus.sone.text.PlainTextPart;
import net.pterodactylus.sone.text.PostPart;
import net.pterodactylus.sone.text.SonePart;
import net.pterodactylus.sone.text.SoneTextParser;
import net.pterodactylus.sone.text.SoneTextParserContext;
import net.pterodactylus.util.template.Filter;
import net.pterodactylus.util.template.Template;
import net.pterodactylus.util.template.TemplateContext;
import net.pterodactylus.util.template.TemplateContextFactory;
import net.pterodactylus.util.template.TemplateParser;

/**
 * {@link Filter} implementation that renders an {@link Iterable} (such as a {@link PartContainer}) of {@link Part}s to HTML.
 *
 * @author <a href="mailto:bombe@pterodactylus.net">David ‘Bombe’ Roden</a>
 */
public class RenderFilter implements Filter {

	private static final Template plainTextTemplate = TemplateParser.parse(new StringReader("<%text|html>"));
	private static final Template linkTemplate =
			TemplateParser.parse(new StringReader("<a class=\"<%cssClass|html>\" href=\"<%link|html>\" title=\"<%title|html>\"><%text|html></a>"));
	private final Core core;
	private final TemplateContextFactory templateContextFactory;

	public RenderFilter(Core core, TemplateContextFactory templateContextFactory) {
		this.core = core;
		this.templateContextFactory = templateContextFactory;
	}

	@Override
	public Object format(TemplateContext templateContext, Object data, Map<String, Object> parameters) {
		if (!(data instanceof Iterable<?>)) {
			return null;
		}
		Iterable<Part> parts = (Iterable<Part>) data;
		int length = parseInt(valueOf(parameters.get("length")), -1);
		int cutOffLength = parseInt(valueOf(parameters.get("cut-off-length")), length);
		StringWriter parsedTextWriter = new StringWriter();
		if (length > -1) {
			int allPartsLength = 0;
			List<Part> shortenedParts = new ArrayList<Part>();
			for (Part part : parts) {
				if (part instanceof PlainTextPart) {
					String longText = part.getText();
					if (allPartsLength < cutOffLength) {
						if ((allPartsLength + longText.length()) > cutOffLength) {
							shortenedParts.add(new PlainTextPart(longText.substring(0, cutOffLength - allPartsLength) + "…"));
						} else {
							shortenedParts.add(part);
						}
					}
					allPartsLength += longText.length();
				} else if (part instanceof LinkPart) {
					if (allPartsLength < cutOffLength) {
						shortenedParts.add(part);
					}
					allPartsLength += part.getText().length();
				} else {
					if (allPartsLength < cutOffLength) {
						shortenedParts.add(part);
					}
				}
			}
			if (allPartsLength >= length) {
				parts = shortenedParts;
			}
		}
		render(parsedTextWriter, parts);
		return parsedTextWriter.toString();
	}

	private void render(Writer writer, Iterable<Part> parts) {
		for (Part part : parts) {
			render(writer, part);
		}
	}

	private void render(Writer writer, Part part) {
		if (part instanceof PlainTextPart) {
			render(writer, (PlainTextPart) part);
		} else if (part instanceof FreenetLinkPart) {
			render(writer, (FreenetLinkPart) part);
		} else if (part instanceof LinkPart) {
			render(writer, (LinkPart) part);
		} else if (part instanceof SonePart) {
			render(writer, (SonePart) part);
		} else if (part instanceof PostPart) {
			render(writer, (PostPart) part);
		} else if (part instanceof AlbumPart) {
			render(writer, (AlbumPart) part);
		} else if (part instanceof Iterable<?>) {
			render(writer, (Iterable<Part>) part);
		}
	}

	private void render(Writer writer, PlainTextPart plainTextPart) {
		TemplateContext templateContext = templateContextFactory.createTemplateContext();
		templateContext.set("text", plainTextPart.getText());
		plainTextTemplate.render(templateContext, writer);
	}

	private void render(Writer writer, FreenetLinkPart freenetLinkPart) {
		renderLink(writer, "/" + freenetLinkPart.getLink(), freenetLinkPart.getText(), freenetLinkPart.getTitle(),
				freenetLinkPart.isTrusted() ? "freenet-trusted" : "freenet");
	}

	private void render(Writer writer, LinkPart linkPart) {
		try {
			renderLink(writer, "/external-link/?_CHECKED_HTTP_=" + URLEncoder.encode(linkPart.getLink(), "UTF-8"), linkPart.getText(),
					linkPart.getTitle(), "internet");
		} catch (UnsupportedEncodingException uee1) {
			/* not possible for UTF-8. */
			throw new RuntimeException("The JVM does not support UTF-8 encoding!", uee1);
		}
	}

	private void render(Writer writer, SonePart sonePart) {
		Sone sone = sonePart.getSone();
		if ((sone != null) && (sone.getName() != null)) {
			String niceName = SoneAccessor.getNiceName(sone);
			renderLink(writer, "viewSone.html?sone=" + sone.getId(), niceName, niceName, "in-sone");
		} else {
			renderLink(writer, "/WebOfTrust/ShowIdentity?id=" + sone.getId(), sone.getId(), sone.getId(), "in-sone");
		}
	}

	private void render(Writer writer, PostPart postPart) {
		SoneTextParser parser = new SoneTextParser(core, core, core);
		SoneTextParserContext parserContext = new SoneTextParserContext(null, postPart.getPost().getSone());
		try {
			Iterable<Part> parts = parser.parse(parserContext, new StringReader(postPart.getPost().getText()));
			StringBuilder excerpt = new StringBuilder();
			for (Part part : parts) {
				excerpt.append(part.getText());
				if (excerpt.length() > 20) {
					int lastSpace = excerpt.lastIndexOf(" ", 20);
					if (lastSpace > -1) {
						excerpt.setLength(lastSpace);
					} else {
						excerpt.setLength(20);
					}
					excerpt.append("…");
					break;
				}
			}
			renderLink(writer, "viewPost.html?post=" + postPart.getPost().getId(), excerpt.toString(),
					SoneAccessor.getNiceName(postPart.getPost().getSone()), postPart.usesDeprecatedLink() ? "internet" : "in-sone");
		} catch (IOException ioe1) {
			/* StringReader shouldn’t throw. */
		}
	}

	private void render(Writer writer, AlbumPart albumPart) {
		Album album = albumPart.getAlbum();
		renderLink(writer, String.format("imageBrowser.html?album=%s", album.getId()), album.getTitle(), album.getDescription(), "in-sone");
	}

	private void renderLink(Writer writer, String link, String text, String title, String cssClass) {
		TemplateContext templateContext = templateContextFactory.createTemplateContext();
		templateContext.set("cssClass", cssClass);
		templateContext.set("link", link);
		templateContext.set("text", text);
		templateContext.set("title", title);
		linkTemplate.render(templateContext, writer);
	}

}
